# Copyright Blair C. Armstrong, September 2008
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#Requires Python 2.4.X

import re
import os
import sys
import time
from numpy import *


errorLog = open("errorLog.txt", "w")

#constants that don't need to be recompiled in each loop
wordRegEx = re.compile("(<DIV CLASS='headword'>)(?P<word>[^<]*)(<SUP>(?P<meaningNum>\d)?</SUP>)?(<DIV STYLE='font-size: small;'>)")
DefinitionRegEx = re.compile("Definition</span></TD><TD  ALIGN='right' VALIGN='baseline' WIDTH='5%'><SPAN class='fieldValue' style='font-size: 80%; font-weight: bold;'>(?P<EntryNum>[\d]+)\.</SPAN></TD><TD  ALIGN='left' VALIGN='baseline' WIDTH='70%'><span class=\"fieldValue\" STYLE=\"font-weight: normal;\">(?P<Defn>[^<]+)</span></TD></TR>")
PartOfSpeechRegEx= re.compile("</A>Part of Speech<A NAME='POSINT[\d]+'></A></span></TD><TD  ALIGN='right' VALIGN='baseline' WIDTH='5%'>&nbsp;</TD><TD  ALIGN='left' VALIGN='baseline' WIDTH='70%'><span class=\"fieldValue\" STYLE=\"color: #CC0000; font-size: 95%;\">(?P<PartOfSpeech>[\w\s]*)</span>")
newlines = re.compile("[\r\n]")
phrase = re.compile(" ")
noWhite= re.compile("[\s';&()/,.^]")

#entries will be triples, word, meaning, senses, and Parts of Speech (POS) information
RoddMeasures = []

pageDir = "./retrievedPages/"

#read in the file that lists all hte parts of speech in wordsmyth
POSList = open("PartsOfSpeech.txt","r")
allPOS = []

for line in POSList:
    line = line.strip()
    allPOS.append(line)

#for every wordsmyth file...
for filename in os.listdir(pageDir):

    #in case you want to break it into smaller folders, adjust the last number
    resultsDir = "processedWords_Folder"+str(len(RoddMeasures)/100000)
    if os.path.isdir(resultsDir):
        #do nothing.
        print "Save directory exists"
    else:
        os.mkdir(resultsDir)


    file = open(pageDir+filename,"r")
    contents = file.read()

    #set the default error message... to be printed if no word is found
    word = "ERROR1: Word was not reset!" + filename

    print "Current file: " + pageDir+filename
    sys.stdout.flush()

    #first, retrieve the word defined in the document
    matchWord = wordRegEx.search(contents)

    if matchWord == None:
        print "ERROR: Unable to find a word entry in this file..." + word +" in file: "+ filename
        errorLog.write(filename+ " ERROR: Unable to find a word entry in this file..." + word +" in file: "+ filename+"\n")
    else:
        word = matchWord.group('word')
        print "Current Word being Processed: " + word

        isPhrase = phrase.search(word)

        #determine if it's a single word or a multiword phrase
        p="singleword."
        if isPhrase:
            p="phrase."

        #replace any whitspace in the word with an underscore
        word_noWhite = noWhite.sub("_", word)


        #next, get the meaning # assicated with this particular meaning, or generate it
        meaningNumber = 1
        if matchWord.group('meaningNum'):
            meaningNumber = matchWord.group('meaningNum')
            print "Using meaning Number in file: " + meaningNumber
        else:
            print "No Meaning Number in file, setting to 1"


        #next, we extract the parts of speech associated with this meaning of the word
        POS = PartOfSpeechRegEx.finditer(contents)

        POSArray = []
        for match in POS:
            POSArray.append(match.group('PartOfSpeech'))

        if len(POSArray) == 0:
            print "ERROR: No Parts of Speech found for word;/meaning: " + word + str(meaningNumber)+ " in file: "+filename
            errorLog.write(filename+ " ERROR: No Parts of Speech found for word/meaning: " + word + str(meaningNumber)+ " in file: "+filename+"\n")

        #Now to grab each definition...
        Definitions = DefinitionRegEx.finditer(contents)

        DefinitionArray = []
        for entry in Definitions:
            cleanDef = entry.group('Defn')
            cleanDef = newlines.sub(" ", cleanDef)

            DefinitionArray.append([entry.group('EntryNum'), cleanDef])

        if len(DefinitionArray) == 0:
            print "ERROR: No Definitions found for word;/meaning: "+" in file: " +filename
            errorLog.write(filename + "\ERROR: No Definitions found for word;/meaning: "+" in file: " +filename+"\n")
        else:
            numDefinitions = len(DefinitionArray)


            #Prepare the array which will hold the distribution information for the word in the current file
            POSDist = zeros(20, int)

            for entry in POSArray:
                POSStr = noWhite.sub("_", entry)

                #get the index in the distribution array where I'll need to add some infiormation
                POSDist[allPOS.index(POSStr)] = POSDist[allPOS.index(POSStr)]+ 1

                writer = open(resultsDir+"/"+word_noWhite+"."+p+str(meaningNumber)+"."+POSStr+".txt","w")

                #also write all of the definitions of a word to a single overall file...
                allMeanings = open(resultsDir+"/"+word_noWhite+"."+p+"all.txt","a")

                print "Finding definitions for meaning: "+POSStr

                #you'll always need to write the first definition...
                writer.write(DefinitionArray[0][1]+"\n")
                allMeanings.write(DefinitionArray[0][1]+"\n")

                #trim that entry from the list...
                DefinitionArray=DefinitionArray[1:]

                #flag to determine when you switch to a new Part of Speech
                flag = 0
                samePOS=True
                for i in range(0,len(DefinitionArray)):
                    if int(DefinitionArray[i][0]) !=1 & samePOS==True:
                        writer.write(DefinitionArray[i][1]+"\n")
                        allMeanings.write(DefinitionArray[i][1]+"\n")
                        POSDist[allPOS.index(POSStr)] = POSDist[allPOS.index(POSStr)]+ 1
                        flag = flag+1
                    else:
                        samePOS =False

                DefinitionArray = DefinitionArray[flag:]
                writer.flush()

            #Write all of the relevant info for this page to the word/meaning/sense array

            #if it's a new word or not...
            new = True
            for i in range(0,len(RoddMeasures)):
                if RoddMeasures[i][0] == word_noWhite:
                    new = False
                    #add that there is one more meaning associated with the word
                    RoddMeasures[i][1] = RoddMeasures[i][1]+1
                    #add the number of senses
                    RoddMeasures[i][2] = RoddMeasures[i][2]+numDefinitions

                    for j in range(0,len(POSDist)):
                        RoddMeasures[i][4][j]= int(RoddMeasures[i][4][j]) + int(POSDist[j])

                    if RoddMeasures[i] != p:
                        print "ERROR: This word is both a noun and a phrase in wordsmyth: " + word + "in file: " + filename
                        errorLog.write(filename+ " ERROR: This word is both a noun and a phrase in wordsmyth: " + word + "in file: " + filename)

            #if it's nowhere in the array, then it's new...
            if new == True:
                RoddMeasures.append([word_noWhite,1,numDefinitions, p, POSDist])



            print "New word information added.  Total unique words: " + str(len(RoddMeasures)) +"\n"

            print time.ctime()
            sys.stdout.flush()

            #done for this word, now start the next word...




#At the very end, print out Rodd's data
RoddFile = open("Word_meaning_sense_POS_cfRoddetal2002.txt","w")

for entry in RoddMeasures:
    RoddFile.write(entry[0]+"\t"+str(entry[1])+"\t"+str(entry[2])+"\t"+entry[3]+"\t")

    for i in range(0,len(entry[4])):
        RoddFile.write(str(entry[4][i])+"\t")
    RoddFile.write("\n")

RoddFile.flush()


print "Number of parts of speech:" +str(len(allPOS))
print "All parts of speech:"
for entry in allPOS:
    print entry
